"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.obfuscateCssFiles = void 0;
const tslib_1 = require("tslib");
const node_fs_1 = tslib_1.__importDefault(require("node:fs"));
const node_path_1 = tslib_1.__importDefault(require("node:path"));
const css_seasoning_1 = require("css-seasoning");
const lightningcss_wasm_1 = tslib_1.__importStar(require("lightningcss-wasm"));
const utils_1 = require("../utils");
const obfuscateCss = ({ cssPath, removeOriginalCss, isFullObfuscation, outCssPath, conversionTables, mode = "random", prefix, suffix, ignorePatterns, generatorSeed = Math.random().toString().slice(2, 10), }) => {
    if (!outCssPath) {
        outCssPath = cssPath;
    }
    else if (!node_fs_1.default.existsSync(node_path_1.default.dirname(outCssPath))) {
        node_fs_1.default.mkdirSync(node_path_1.default.dirname(outCssPath));
    }
    const cssContent = node_fs_1.default.readFileSync(cssPath, "utf-8");
    let transformerMode = mode === "simplify" ? "minimal" : "hash";
    if (!transformerMode) {
        if (mode === "simplify-seedable") {
            (0, utils_1.log)("warn", "CSS obfuscation", "The 'simplify-seedable' mode is deprecated, please use 'random' or 'simplify' instead. Now will fall back to 'random' mode.");
            transformerMode = "hash";
        }
        (0, utils_1.log)("error", "CSS obfuscation", `Invalid mode: ${mode}`);
        throw new Error(`Invalid mode: ${mode}`);
    }
    let finCss = "";
    const { css: obfuscatedCss, conversionTables: newConversionTables } = (0, css_seasoning_1.transform)({
        css: cssContent,
        conversionTables: conversionTables,
        mode: transformerMode,
        prefix: prefix,
        suffix: suffix,
        seed: (0, utils_1.stringToNumber)(generatorSeed),
        ignorePatterns: ignorePatterns,
    });
    if (removeOriginalCss) {
        finCss = obfuscatedCss;
    }
    else {
        const mixedCss = cssContent + obfuscatedCss;
        const { code } = (0, lightningcss_wasm_1.transform)({
            filename: "style.css",
            code: new TextEncoder().encode(mixedCss),
            minify: true,
        });
        finCss = new TextDecoder().decode(code);
    }
    const totalConversion = Object.keys(newConversionTables.selectors).length +
        Object.keys(newConversionTables.idents).length;
    if (removeOriginalCss) {
        (0, utils_1.log)("info", "CSS rules:", `Modified ${totalConversion} CSS rules to ${(0, utils_1.getFilenameFromPath)(cssPath)}`);
    }
    else {
        const oldTotalConversion = conversionTables
            ? Object.keys(conversionTables.selectors).length +
                Object.keys(conversionTables.idents).length
            : 0;
        (0, utils_1.log)("info", "CSS rules:", `Added ${totalConversion - oldTotalConversion} new CSS rules to ${(0, utils_1.getFilenameFromPath)(cssPath)}`);
    }
    const sizeBefore = Buffer.byteLength(cssContent, "utf8");
    node_fs_1.default.writeFileSync(outCssPath, finCss);
    const sizeAfter = Buffer.byteLength(finCss, "utf8");
    const percentChange = Math.round((sizeAfter / sizeBefore) * 100);
    (0, utils_1.log)("success", "CSS obfuscated:", `Size from ${sizeBefore} to ${sizeAfter} bytes (${percentChange}%) in ${(0, utils_1.getFilenameFromPath)(cssPath)}`);
    return {
        conversionTables: newConversionTables,
    };
};
const obfuscateCssFiles = ({ selectorConversionJsonFolderPath, buildFolderPath, whiteListedFolderPaths = [], blackListedFolderPaths = [], mode = "random", prefix, suffix, ignorePatterns, generatorSeed = new Date().getTime().toString(), removeOriginalCss = false, }) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    yield Promise.all([(0, css_seasoning_1.initTransform)(), (0, lightningcss_wasm_1.default)()]);
    if (!node_fs_1.default.existsSync(selectorConversionJsonFolderPath)) {
        node_fs_1.default.mkdirSync(selectorConversionJsonFolderPath);
    }
    const conversionTables = (0, utils_1.loadConversionTables)(selectorConversionJsonFolderPath);
    const cssPaths = (0, utils_1.findAllFilesWithExt)(".css", buildFolderPath, {
        whiteListedFolderPaths,
        blackListedFolderPaths,
    });
    const tables = {
        selectors: {},
        idents: {},
    };
    cssPaths.forEach((cssPath) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const { conversionTables: newConversionTables } = yield obfuscateCss({
            cssPath: cssPath,
            conversionTables: conversionTables,
            prefix,
            suffix,
            mode,
            ignorePatterns,
            generatorSeed,
            removeOriginalCss,
        });
        Object.entries(newConversionTables.selectors).forEach(([key, value]) => {
            if (!tables.selectors[key]) {
                tables.selectors[key] = value;
            }
        });
        Object.entries(newConversionTables.idents).forEach(([key, value]) => {
            if (!tables.idents[key]) {
                tables.idents[key] = value;
            }
        });
    }));
    return {
        conversionTables: tables,
    };
});
exports.obfuscateCssFiles = obfuscateCssFiles;
