"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.searchStringLiterals = exports.obfuscateJsWithAst = void 0;
const tslib_1 = require("tslib");
const generator_1 = tslib_1.__importDefault(require("@babel/generator"));
const parser = tslib_1.__importStar(require("@babel/parser"));
const traverse_1 = tslib_1.__importDefault(require("@babel/traverse"));
const t = tslib_1.__importStar(require("@babel/types"));
const utils_1 = require("../utils");
const obfuscateJsWithAst = (code, selectorConversion, startingKeys = [], stripUnnecessarySpace = true) => {
    const ast = parser.parse(code, { sourceType: "module", plugins: ["jsx"] });
    const usedKeys = new Set();
    (0, traverse_1.default)(ast, {
        ObjectProperty(path) {
            if (t.isIdentifier(path.node.key) && path.node.key.name === "className") {
                (0, exports.searchStringLiterals)(path.get("value"), (str) => {
                    if (startingKeys.length > 0 && !startingKeys.includes(str)) {
                        return str;
                    }
                    if (!selectorConversion) {
                        return "{{obfuscated}}";
                    }
                    const newStr = stripUnnecessarySpace
                        ? str
                            .replace(/\s+/g, " ")
                            .trimStart()
                        : str;
                    const { obfuscatedContent, usedKeys: obfuscateUsedKeys } = (0, utils_1.obfuscateKeys)(selectorConversion, newStr);
                    if (obfuscatedContent !== newStr) {
                        obfuscateUsedKeys.forEach((key) => usedKeys.add(key));
                        return obfuscatedContent;
                    }
                });
            }
        },
    });
    const options = {
        compact: true,
        concise: true,
        retainLines: false,
        comments: false,
        minified: true,
    };
    const obfuscatedCode = (0, generator_1.default)(ast, options, code);
    return {
        obfuscatedCode: obfuscatedCode.code,
        usedKeys: usedKeys,
    };
};
exports.obfuscateJsWithAst = obfuscateJsWithAst;
const searchStringLiterals = (path, callback, scannedNodes = new Set()) => {
    if (path.node && scannedNodes.has(path.node)) {
        return;
    }
    scannedNodes.add(path.node);
    if (t.isBlockStatement(path.node)) {
        const body = path.get("body");
        if (Array.isArray(body)) {
            body.forEach((nodePath) => {
                switch (nodePath.node.type) {
                    case "ReturnStatement":
                    case "IfStatement":
                    case "SwitchStatement":
                    case "ExpressionStatement":
                    case "ForStatement":
                    case "WhileStatement":
                    case "TryStatement":
                        (0, exports.searchStringLiterals)(nodePath, callback, scannedNodes);
                        break;
                }
            });
        }
        else {
            (0, exports.searchStringLiterals)(body, callback, scannedNodes);
        }
    }
    else if (t.isReturnStatement(path.node)) {
        const argument = path.get("argument");
        if (argument) {
            if (!Array.isArray(argument)) {
                (0, exports.searchStringLiterals)(argument, callback, scannedNodes);
            }
            else {
                argument.forEach((arg) => {
                    (0, exports.searchStringLiterals)(arg, callback, scannedNodes);
                });
            }
        }
    }
    else if (t.isBinaryExpression(path.node)) {
        const left = path.get("left");
        const right = path.get("right");
        if (left && !Array.isArray(left)) {
            (0, exports.searchStringLiterals)(left, callback, scannedNodes);
        }
        if (right && !Array.isArray(right)) {
            (0, exports.searchStringLiterals)(right, callback, scannedNodes);
        }
    }
    else if (t.isStringLiteral(path.node)) {
        const replacement = callback(path.node.value);
        if (replacement) {
            path.replaceWith(t.stringLiteral(replacement));
        }
    }
    else if (t.isIdentifier(path.node)) {
        const variableName = path.node.name;
        const binding = path.scope.getBinding(variableName);
        if (binding && t.isVariableDeclarator(binding.path.node)) {
            const init = binding.path.get("init");
            if (init && !Array.isArray(init)) {
                (0, exports.searchStringLiterals)(init, callback, scannedNodes);
            }
        }
        else if (binding && t.isFunctionDeclaration(binding.path.node)) {
            const body = binding.path.get("body");
            if (body && !Array.isArray(body)) {
                (0, exports.searchStringLiterals)(body, callback, scannedNodes);
            }
        }
    }
    else if (t.isCallExpression(path.node)) {
        const callee = path.get("callee");
        if (callee && !Array.isArray(callee)) {
            (0, exports.searchStringLiterals)(callee, callback, scannedNodes);
        }
        const args = path.get("arguments");
        if (Array.isArray(args)) {
            args.forEach((arg) => {
                if (t.isStringLiteral(arg.node)) {
                    const replacement = callback(arg.node.value);
                    if (replacement) {
                        arg.replaceWith(t.stringLiteral(replacement));
                    }
                }
                else {
                    (0, exports.searchStringLiterals)(arg, callback, scannedNodes);
                }
            });
        }
    }
    else if (t.isConditionalExpression(path.node)) {
        const test = path.get("test");
        const consequent = path.get("consequent");
        const alternate = path.get("alternate");
        if (test && !Array.isArray(test)) {
            (0, exports.searchStringLiterals)(test, callback, scannedNodes);
        }
        if (consequent && !Array.isArray(consequent)) {
            (0, exports.searchStringLiterals)(consequent, callback, scannedNodes);
        }
        if (alternate && !Array.isArray(alternate)) {
            (0, exports.searchStringLiterals)(alternate, callback, scannedNodes);
        }
    }
    else if (t.isIfStatement(path.node)) {
        const test = path.get("test");
        const consequent = path.get("consequent");
        const alternate = path.get("alternate");
        if (test && !Array.isArray(test)) {
            (0, exports.searchStringLiterals)(test, callback, scannedNodes);
        }
        if (consequent && !Array.isArray(consequent)) {
            (0, exports.searchStringLiterals)(consequent, callback, scannedNodes);
        }
        if (alternate && !Array.isArray(alternate)) {
            (0, exports.searchStringLiterals)(alternate, callback, scannedNodes);
        }
    }
    else if (t.isObjectExpression(path.node)) {
        const properties = path.get("properties");
        if (Array.isArray(properties)) {
            properties.forEach((prop) => {
                (0, exports.searchStringLiterals)(prop, callback, scannedNodes);
            });
        }
    }
    else if (t.isObjectProperty(path.node)) {
        const value = path.get("value");
        if (value && !Array.isArray(value)) {
            (0, exports.searchStringLiterals)(value, callback, scannedNodes);
        }
    }
    else if (t.isArrayExpression(path.node)) {
        const elements = path.get("elements");
        if (Array.isArray(elements)) {
            elements.forEach((element) => {
                (0, exports.searchStringLiterals)(element, callback, scannedNodes);
            });
        }
    }
    else if (t.isSwitchStatement(path.node)) {
        const cases = path.get("cases");
        if (Array.isArray(cases)) {
            cases.forEach((c) => {
                (0, exports.searchStringLiterals)(c, callback, scannedNodes);
            });
        }
    }
    else if (t.isSwitchCase(path.node)) {
        const consequent = path.get("consequent");
        if (Array.isArray(consequent)) {
            consequent.forEach((c) => {
                if (t.isReturnStatement(c.node)) {
                    (0, exports.searchStringLiterals)(c, callback, scannedNodes);
                }
            });
        }
    }
    else if (t.isFunctionDeclaration(path.node)) {
        const body = path.get("body");
        if (body && !Array.isArray(body)) {
            (0, exports.searchStringLiterals)(body, callback, scannedNodes);
        }
    }
    else if (t.isForStatement(path.node)) {
        const body = path.get("body");
        if (body && !Array.isArray(body)) {
            (0, exports.searchStringLiterals)(body, callback, scannedNodes);
        }
    }
    else if (t.isExpressionStatement(path.node)) {
        const expression = path.get("expression");
        if (expression && !Array.isArray(expression)) {
            (0, exports.searchStringLiterals)(expression, callback, scannedNodes);
        }
    }
    else if (t.isAssignmentExpression(path.node)) {
        const right = path.get("right");
        if (right && !Array.isArray(right)) {
            (0, exports.searchStringLiterals)(right, callback, scannedNodes);
        }
    }
    else if (t.isWhileStatement(path.node)) {
        const body = path.get("body");
        if (body && !Array.isArray(body)) {
            (0, exports.searchStringLiterals)(body, callback, scannedNodes);
        }
    }
    else if (t.isSpreadElement(path.node)) {
        const argument = path.get("argument");
        if (argument && !Array.isArray(argument)) {
            (0, exports.searchStringLiterals)(argument, callback, scannedNodes);
        }
    }
    else if (t.isArrowFunctionExpression(path.node)) {
        const body = path.get("body");
        if (body && !Array.isArray(body)) {
            (0, exports.searchStringLiterals)(body, callback, scannedNodes);
        }
    }
    else if (t.isTryStatement(path.node)) {
        const block = path.get("block");
        const handler = path.get("handler");
        if (block && !Array.isArray(block)) {
            (0, exports.searchStringLiterals)(block, callback, scannedNodes);
        }
        if (handler && !Array.isArray(handler)) {
            const handlerBody = handler.get("body");
            if (handlerBody && !Array.isArray(handlerBody)) {
                (0, exports.searchStringLiterals)(handlerBody, callback, scannedNodes);
            }
        }
    }
    else if (t.isMemberExpression(path.node)) {
        const object = path.get("object");
        const property = path.get("property");
        const argument = path.get("argument");
        if (object && !Array.isArray(object)) {
            (0, exports.searchStringLiterals)(object, callback, scannedNodes);
        }
        if (property && !Array.isArray(property)) {
            (0, exports.searchStringLiterals)(property, callback, scannedNodes);
        }
        if (argument && !Array.isArray(argument)) {
            (0, exports.searchStringLiterals)(argument, callback, scannedNodes);
        }
        else if (Array.isArray(argument)) {
            argument.forEach((arg) => {
                (0, exports.searchStringLiterals)(arg, callback, scannedNodes);
            });
        }
    }
    else if (t.isTemplateLiteral(path.node)) {
        const quasis = path.get("quasis");
        const expressions = path.get("expressions");
        if (Array.isArray(quasis)) {
            quasis.forEach((quasi) => {
                (0, exports.searchStringLiterals)(quasi, callback, scannedNodes);
            });
        }
        if (Array.isArray(expressions)) {
            expressions.forEach((expression) => {
                (0, exports.searchStringLiterals)(expression, callback, scannedNodes);
            });
        }
    }
    else if (t.isTemplateElement(path.node)) {
        const node = path.node;
        if (node.value) {
            const { raw, cooked } = node.value;
            if (raw) {
                const rawReplacement = callback(raw);
                if (rawReplacement !== undefined) {
                    node.value.raw = rawReplacement;
                }
            }
            if (cooked) {
                const cookedReplacement = callback(cooked);
                if (cookedReplacement !== undefined) {
                    node.value.cooked = cookedReplacement;
                }
            }
        }
    }
    else {
        path.traverse({
            Identifier(innerPath) {
                (0, exports.searchStringLiterals)(innerPath, callback, scannedNodes);
            },
        });
    }
    return path;
};
exports.searchStringLiterals = searchStringLiterals;
