"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadConversionTables = exports.stringToNumber = exports.duplicationCheck = exports.replaceFirstMatch = exports.findAllFilesWithExt = exports.addKeysToRegistery = exports.findContentBetweenMarker = exports.findClosestSymbolPosition = exports.normalizePath = exports.getFilenameFromPath = exports.obfuscateKeys = exports.replaceJsonKeysInFiles = exports.usedKeyRegistery = exports.setLogLevel = exports.log = void 0;
const tslib_1 = require("tslib");
const node_fs_1 = tslib_1.__importDefault(require("node:fs"));
const node_path_1 = tslib_1.__importDefault(require("node:path"));
const css_seasoning_1 = require("css-seasoning");
const html_1 = require("./handlers/html");
const js_1 = require("./handlers/js");
const shouldIncludePath = (filePath, whiteListedFolderPaths = [], blackListedFolderPaths = []) => {
    const normalizedPath = (0, exports.normalizePath)(filePath);
    const isBlacklisted = blackListedFolderPaths.some((excludePath) => {
        if (typeof excludePath === "string") {
            return normalizedPath.includes(excludePath);
        }
        return excludePath.test(normalizedPath);
    });
    if (isBlacklisted) {
        return false;
    }
    const isWhitelisted = whiteListedFolderPaths.length === 0 ||
        whiteListedFolderPaths.some((includePath) => {
            if (typeof includePath === "string") {
                return normalizedPath.includes(includePath);
            }
            return includePath.test(normalizedPath);
        });
    return isWhitelisted;
};
const issuer = "[next-css-obfuscator]";
let logLevel = "info";
const levels = ["debug", "info", "warn", "error", "success"];
const log = (type, task, data) => {
    if (levels.indexOf(type) < levels.indexOf(logLevel)) {
        return;
    }
    const mainColor = "\x1b[38;2;99;102;241m%s\x1b[0m";
    switch (type) {
        case "debug":
            console.debug(mainColor, issuer, "[Debug] \x1b[37m", task, data, "\x1b[0m");
            break;
        case "info":
            console.info(mainColor, issuer, "🗯️ \x1b[36m", task, data, "\x1b[0m");
            break;
        case "warn":
            console.warn(mainColor, issuer, "⚠️ \x1b[33m", task, data, "\x1b[0m");
            break;
        case "error":
            console.error(mainColor, issuer, "⛔ \x1b[31m", task, data, "\x1b[0m");
            break;
        case "success":
            console.log(mainColor, issuer, "✅ \x1b[32m", task, data, "\x1b[0m");
            break;
        default:
            console.log("'\x1b[0m'", issuer, task, data, "\x1b[0m");
            break;
    }
};
exports.log = log;
const setLogLevel = (level) => {
    logLevel = level;
};
exports.setLogLevel = setLogLevel;
exports.usedKeyRegistery = new Set();
const replaceJsonKeysInFiles = ({ conversionTables, targetFolder, allowExtensions, contentIgnoreRegexes, whiteListedFolderPaths, blackListedFolderPaths, enableObfuscateMarkerClasses, obfuscateMarkerClasses, removeObfuscateMarkerClassesAfterObfuscated, enableJsAst, }) => {
    if (removeObfuscateMarkerClassesAfterObfuscated) {
        obfuscateMarkerClasses.forEach((obfuscateMarkerClass) => {
            conversionTables.selectors[(0, css_seasoning_1.cssEscape)(`.${obfuscateMarkerClass}`)] = "";
        });
    }
    const cssPaths = [];
    const replaceJsonKeysInFile = (filePath) => {
        const fileExt = node_path_1.default.extname(filePath).toLowerCase();
        if (node_fs_1.default.statSync(filePath).isDirectory()) {
            node_fs_1.default.readdirSync(filePath).forEach((subFilePath) => {
                replaceJsonKeysInFile(node_path_1.default.join(filePath, subFilePath));
            });
        }
        else if (allowExtensions.includes(fileExt)) {
            if (!shouldIncludePath(filePath, whiteListedFolderPaths, blackListedFolderPaths)) {
                return;
            }
            let fileContent = node_fs_1.default.readFileSync(filePath, "utf-8");
            const fileContentOriginal = fileContent;
            if (enableObfuscateMarkerClasses) {
                obfuscateMarkerClasses.forEach((obfuscateMarkerClass) => {
                    const isHtml = [".html"].includes(fileExt);
                    if (isHtml) {
                        const htmlRegex = new RegExp("(<(.*)>(.*)</([^br][A-Za-z0-9]+)>)", "g");
                        const htmlMatch = fileContent.match(htmlRegex);
                        if (htmlMatch) {
                            const htmlOriginal = htmlMatch[0];
                            const { obfuscatedContent, usedKeys } = (0, html_1.obfuscateHtmlClassNames)({
                                html: htmlOriginal,
                                selectorConversion: conversionTables.selectors,
                                obfuscateMarkerClass: obfuscateMarkerClass,
                                contentIgnoreRegexes: contentIgnoreRegexes,
                            });
                            (0, exports.addKeysToRegistery)(usedKeys);
                            if (htmlOriginal !== obfuscatedContent) {
                                fileContent = fileContent.replace(htmlOriginal, obfuscatedContent);
                            }
                        }
                    }
                    else {
                        const obfuscateScriptContent = (0, js_1.obfuscateJs)(fileContent, obfuscateMarkerClass, conversionTables.selectors, filePath, contentIgnoreRegexes, enableJsAst);
                        if (fileContent !== obfuscateScriptContent) {
                            fileContent = obfuscateScriptContent;
                            (0, exports.log)("debug", "Obscured keys in JS like content file:", (0, exports.normalizePath)(filePath));
                        }
                    }
                });
            }
            else {
                if ([".js"].includes(fileExt)) {
                    const obfuscateScriptContent = (0, js_1.obfuscateJs)(fileContent, enableJsAst ? "" : "jsx", conversionTables.selectors, filePath, contentIgnoreRegexes, enableJsAst);
                    if (fileContent !== obfuscateScriptContent) {
                        fileContent = obfuscateScriptContent;
                        (0, exports.log)("debug", "Obscured keys in JSX related file:", (0, exports.normalizePath)(filePath));
                    }
                }
                else if ([".html"].includes(fileExt)) {
                    const { obfuscatedContent, usedKeys } = (0, html_1.obfuscateHtmlClassNames)({
                        html: fileContent,
                        selectorConversion: conversionTables.selectors,
                        contentIgnoreRegexes: contentIgnoreRegexes,
                    });
                    fileContent = obfuscatedContent;
                    (0, exports.addKeysToRegistery)(usedKeys);
                }
                else {
                    const { obfuscatedContent, usedKeys } = (0, exports.obfuscateKeys)(conversionTables.selectors, fileContent, contentIgnoreRegexes);
                    fileContent = obfuscatedContent;
                    (0, exports.addKeysToRegistery)(usedKeys);
                }
            }
            if (fileContentOriginal !== fileContent) {
                (0, exports.log)("success", "Data obfuscated:", (0, exports.normalizePath)(filePath));
                node_fs_1.default.writeFileSync(filePath, fileContent);
            }
        }
        else if (fileExt === ".css") {
            cssPaths.push(filePath);
        }
    };
    replaceJsonKeysInFile(targetFolder);
};
exports.replaceJsonKeysInFiles = replaceJsonKeysInFiles;
const obfuscateKeys = (selectorConversion, fileContent, contentIgnoreRegexes = []) => {
    const usedKeys = new Set();
    Object.keys(selectorConversion).forEach((key) => {
        const fileContentOriginal = fileContent;
        let keyUse = (0, css_seasoning_1.cssUnescape)(key).slice(1);
        keyUse = escapeRegExp(keyUse.replace(/\\/g, ""));
        const exactMatchRegex = new RegExp(`([\\s"'\\\`]|^)(${keyUse})(?=$|[\\s"'\\\`]|\\\\n|\\\\",|\\\\"})`, "g");
        const replacement = `$1${selectorConversion[key].slice(1).replace(/\\/g, "").slice(1)}`;
        const matches = fileContent.match(exactMatchRegex);
        const originalObscuredContentPairs = matches === null || matches === void 0 ? void 0 : matches.map((match) => {
            return {
                originalContent: match,
                obscuredContent: match.replace(exactMatchRegex, replacement),
            };
        });
        fileContent = fileContent.replace(exactMatchRegex, replacement);
        if (contentIgnoreRegexes.length > 0) {
            contentIgnoreRegexes.forEach((regex) => {
                const originalContentFragments = fileContentOriginal.match(regex);
                originalContentFragments === null || originalContentFragments === void 0 ? void 0 : originalContentFragments.map((originalContentFragment) => {
                    originalObscuredContentPairs === null || originalObscuredContentPairs === void 0 ? void 0 : originalObscuredContentPairs.map((pair) => {
                        if (originalContentFragments === null || originalContentFragments === void 0 ? void 0 : originalContentFragments.some((fragment) => fragment.includes(pair.originalContent))) {
                            (0, exports.log)("debug", "Obscured keys:", `Ignored ${pair.originalContent} at ${originalContentFragment}`);
                            fileContent = fileContent.replace(originalContentFragment.replace(pair.originalContent, pair.obscuredContent), originalContentFragment);
                        }
                    });
                });
            });
        }
        if (fileContentOriginal !== fileContent && !usedKeys.has(key)) {
            usedKeys.add(key);
        }
    });
    return { obfuscatedContent: fileContent, usedKeys: usedKeys };
};
exports.obfuscateKeys = obfuscateKeys;
const escapeRegExp = (str) => {
    return str.replace(/[.*+?^${}()|[\]\\]/g, "\\$&");
};
const getFilenameFromPath = (filePath) => {
    return filePath.replace(/^.*[\\/]/, "");
};
exports.getFilenameFromPath = getFilenameFromPath;
const normalizePath = (filePath) => {
    return filePath.replace(/\\/g, "/");
};
exports.normalizePath = normalizePath;
const findClosestSymbolPosition = (content, openMarker, closeMarker, startPosition = 0, direction = "backward") => {
    let level = 0;
    let currentPos = startPosition;
    if (direction === "backward") {
        while (currentPos >= 0 && level >= 0) {
            if (content.slice(currentPos, currentPos + openMarker.length) === openMarker) {
                level--;
            }
            else if (content.slice(currentPos, currentPos + closeMarker.length) ===
                closeMarker) {
                level++;
            }
            currentPos--;
        }
        if (level < 0) {
            currentPos += 2;
        }
    }
    else {
        while (currentPos < content.length && level >= 0) {
            if (content.slice(currentPos, currentPos + openMarker.length) === openMarker) {
                level++;
            }
            else if (content.slice(currentPos, currentPos + closeMarker.length) ===
                closeMarker) {
                level--;
            }
            currentPos++;
        }
        if (level < 0) {
            currentPos--;
        }
    }
    return currentPos;
};
exports.findClosestSymbolPosition = findClosestSymbolPosition;
const findContentBetweenMarker = (content, targetStr, openMarker, closeMarker) => {
    if (openMarker === closeMarker) {
        throw new Error("openMarker and closeMarker can not be the same");
    }
    let targetStrPosition = content.indexOf(targetStr);
    const truncatedContents = [];
    while (targetStrPosition !== -1 && targetStrPosition < content.length) {
        const openPos = (0, exports.findClosestSymbolPosition)(content, openMarker, closeMarker, targetStrPosition, "backward");
        const closePos = (0, exports.findClosestSymbolPosition)(content, openMarker, closeMarker, targetStrPosition, "forward");
        if (openPos === -1 && closePos === -1) {
            break;
        }
        if (openPos > -1 && closePos > -1) {
            truncatedContents.push(content.slice(openPos, closePos));
            targetStrPosition = content.indexOf(targetStr, closePos + 1);
        }
        else {
            targetStrPosition = content.indexOf(targetStr, targetStrPosition + 1);
        }
    }
    return truncatedContents;
};
exports.findContentBetweenMarker = findContentBetweenMarker;
const addKeysToRegistery = (usedKeys) => {
    usedKeys.forEach((key) => {
        exports.usedKeyRegistery.add(key);
    });
};
exports.addKeysToRegistery = addKeysToRegistery;
const findAllFilesWithExt = (ext, targetFolderPath, options) => {
    if (!node_fs_1.default.existsSync(targetFolderPath)) {
        return [];
    }
    const targetExtFiles = [];
    const whiteList = (options === null || options === void 0 ? void 0 : options.whiteListedFolderPaths) || [];
    const blackList = (options === null || options === void 0 ? void 0 : options.blackListedFolderPaths) || [];
    const findFiles = (dir) => {
        const files = node_fs_1.default.readdirSync(dir);
        files.forEach((file) => {
            const filePath = node_path_1.default.join(dir, file);
            if (!shouldIncludePath(filePath, whiteList, blackList)) {
                return;
            }
            if (node_fs_1.default.statSync(filePath).isDirectory()) {
                findFiles(filePath);
            }
            else {
                if (file.endsWith(ext)) {
                    targetExtFiles.push(filePath);
                }
            }
        });
    };
    findFiles(targetFolderPath);
    return targetExtFiles;
};
exports.findAllFilesWithExt = findAllFilesWithExt;
const replaceFirstMatch = (source, find, replace) => {
    const index = source.indexOf(find);
    if (index !== -1) {
        return source.slice(0, index) + replace + source.slice(index + find.length);
    }
    return source;
};
exports.replaceFirstMatch = replaceFirstMatch;
const duplicationCheck = (arr) => {
    const set = new Set(arr);
    return arr.length !== set.size;
};
exports.duplicationCheck = duplicationCheck;
const stringToNumber = (str) => {
    return str.split("").reduce((acc, char) => {
        return acc + char.charCodeAt(0);
    }, 0);
};
exports.stringToNumber = stringToNumber;
const loadConversionTables = (folderPath) => {
    const tables = {
        idents: {},
        selectors: {},
    };
    node_fs_1.default.readdirSync(folderPath).forEach((file) => {
        const filePath = node_path_1.default.join(folderPath, file);
        const fileData = JSON.parse(node_fs_1.default.readFileSync(filePath, "utf-8"));
        if (Object.keys(fileData).includes("idents") &&
            Object.keys(fileData).includes("selectors")) {
            Object.assign(tables.idents, fileData.idents);
            Object.assign(tables.selectors, fileData.selectors);
        }
        else {
            Object.assign(tables.selectors, fileData);
        }
    });
    return tables;
};
exports.loadConversionTables = loadConversionTables;
